﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SatikSudokuSolver
{
    public class SudokuSolver
    {
        public SudokuSolver()
        {
        }

        public void Solve(SudokuBoard board)
        {
            while (!board.IsSolved)
            {
                if (!SolveStep(board))
                    return;
            }
        }

        protected bool SolveStep(SudokuBoard board)
        {
            bool anythingPlaced = SolveSimple(board);

            if (!anythingPlaced)
            {
                anythingPlaced = SolveBacktrack(board);
            }

            return anythingPlaced;
        }

        /// <summary>
        /// This is used when there are more possible ways to fill numbers in the board
        /// </summary>
        protected bool SolveBacktrack(SudokuBoard board)
        {
            var allowedNumbers = board.GetAllowedNumbers();

            // find empty field with lowest allowed numbers count
            int min = 10;
            int minX = -1;
            int minY = -1;
            for (byte y = 0; y < 9; y++)
            {
                for (byte x = 0; x < 9; x++)
                {
                    if (allowedNumbers[x, y].Count != 0 && allowedNumbers[x, y].Count < min)
                    {
                        min = allowedNumbers[x, y].Count;
                        minX = x;
                        minY = y;
                    }
                }
            }

            if (min != 10)
            {
                // test all possible inputs
                foreach (byte number in allowedNumbers[minX, minY])
                {
                    // Try one solution on copied board
                    var boardCopy = board.Copy();
                    boardCopy.SetNumber(minX, minY, number);
                    Solve(boardCopy);

                    // if the solution worked, copy the solution to main board
                    if (boardCopy.IsSolved)
                    {
                        board.CopyFrom(boardCopy);
                        return true;
                    }
                }
            }
            else
            {
                return board.IsSolved;
            }

            return false;
        }

        /// <summary>
        /// This step only looks at allowed numbers and if there are any places, where only one number is possible, it is placed in
        /// </summary>
        protected bool SolveSimple(SudokuBoard board)
        {
            var allowedNumbers = board.GetAllowedNumbers();

            bool anyFound = false;

            for (byte y = 0; y < 9; y++)
            {
                for (byte x = 0; x < 9; x++)
                {
                    if (allowedNumbers[x, y].Count() == 1)
                    {
                        board.SetNumber(x, y, allowedNumbers[x, y].First());
                        allowedNumbers[x, y].Clear();
                        anyFound = true;
                    }
                }
            }

            return anyFound;
        }
    }
}
