﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SatikSudokuSolver;


namespace UnitTestProject1
{
  [TestClass]
  public class UnitTest1
  {
    const string boardStringWithSpaces = @"
___1_2___
3_98_41_7
8_______2
_3_4_6_7_
4_______1
_1_2_5_9_
5_______8
1_65_39_4
___6_8___
";

    const string boardStringSolved = @"
674152389
329864157
851397642
932416875
465789231
718235496
543921768
186573924
297648513
";

    const string testBoardVeryEmpty = @"
___1_2___
8_______2
___4___7_
4_______1
_1___5___
________8
1__5_____
_____8___
";

    const string testBoardWithWhiteSpaceChar = @"
___1_2_ _
_
8  ___
____2
_ _     _4__ _7_
4__
__ ___1
_1__   _
5_ __
__  __
_  ___ 8
1__5_
_  ___
__ ___8___
";

    [TestMethod]
    public void SolvedCheckTest()
    {
      SudokuBoard solvedBoard = new SudokuBoard(boardStringSolved);
      Assert.IsTrue(solvedBoard.IsSolved);
    }

    [TestMethod]
    public void GetNumbersInRowTest()
    {
      SudokuBoard solvedBoard = new SudokuBoard(boardStringSolved);
      byte[] expected = { 6, 7, 4, 1, 5, 2, 3, 8, 9 };
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInRow(0).ToList());
    }

    [TestMethod]
    public void GetNumbersInColumnTest()
    {
      SudokuBoard solvedBoard = new SudokuBoard(boardStringSolved);
      byte[] expected = { 6, 3, 8, 9, 4, 7, 5, 1, 2 };
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInColumn(0).ToList());
    }

    [TestMethod]
    public void GetNumbersInSquareTest1()
    {
      SudokuBoard solvedBoard = new SudokuBoard(boardStringSolved);
      byte[] expected = { 6, 7, 4, 3, 2, 9, 8, 5, 1 };
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(0, 0).ToList());
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(2, 2).ToList());
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(0, 2).ToList());
    }

    [TestMethod]
    public void GetNumbersInSquareTest2()
    {
      SudokuBoard solvedBoard = new SudokuBoard(boardStringSolved);
      byte[] expected = { 7, 6, 8, 9, 2, 4, 5, 1, 3 };
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(8, 8).ToList());
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(6, 8).ToList());
      CollectionAssert.AreEqual(expected.ToList(), solvedBoard.GetNumbersInSquare(6, 6).ToList());
    }

    [TestMethod]
    public void SolveSimpleBoardTest()
    {
      SudokuBoard solvingBoard = new SudokuBoard(boardStringWithSpaces);
      SudokuSolver solver = new SudokuSolver();
      solver.Solve(solvingBoard);
      Assert.IsTrue(solvingBoard.IsSolved);
    }

    [TestMethod]
    public void SolveHardBoardTest()
    {
      SudokuBoard solvingBoard = new SudokuBoard(testBoardVeryEmpty);
      SudokuSolver solver = new SudokuSolver();
      solver.Solve(solvingBoard);
      Assert.IsTrue(solvingBoard.IsSolved);
    }

    [TestMethod]
    public void ParseWithSpacesAndSolveHardBoardTest()
    {
      SudokuBoard solvingBoard = new SudokuBoard(testBoardWithWhiteSpaceChar);
      SudokuSolver solver = new SudokuSolver();
      solver.Solve(solvingBoard);
      Assert.IsTrue(solvingBoard.IsSolved);
    }
  }
}
