﻿using System;
using System.Collections.Generic;

namespace Sudoku
{
    /// <summary>
    /// Třída reprezentující komunikátora, který přes konzoli zprostředkovává komunikaci s uživatelem a instancemi
    /// </summary>
    class Communicator
    {
        /// <summary>
        /// Načte sudoku ze standardního vstupu
        /// </summary>
        /// <returns>načtené sudoku</returns>
        public Sudoku Load()
        {
            Console.WriteLine("Vítejte v řešiči Sudoku. Zadejte prosím vaše sudoku postupně po řádcích. Čísla oddělte mezerami, prázdná pole nahradtě znakem podtržítka ('_'):\n\n");
            Sudoku sudoku = new Sudoku(); //nová instance, návratová proměnná
            for (int y = 0; y < 9; y++) //'y' - souřadnice y načítáme (tedy řádky - string[])
            {
                string[] radek = Console.ReadLine().Split(' '); //jako oddělovač slouží mezera
                bool appropriate = true; //je načtený řádek v pořádku?
                if (radek.Length != 9) appropriate = false;
                else
                {
                    for (int r = 0; r < 9; r++)
                    {
                        if (radek[r] == "_") radek[r] = "0"; //nahrazujeme '_' podrtžítka (prázdná místa) nulami, kvůli pozdější konverzi na číslo
                        if (radek[r].Length != 1 || !("0123456789".Contains(radek[r]))) appropriate = false; //ověřujeme platnost
                    }
                }

                if (appropriate) //korektní řádek parsujeme do tabulky sudoku
                {
                    List<int> radekcisel = new List<int>();
                    foreach (string s in radek) radekcisel.Add(int.Parse(s));
                    sudoku.Tabulka[y] = radekcisel.ToArray();
                }
                else //nekorektní řádek - zobrazíme hlášku a vypíšeme dosavadně načtené řádky
                {
                    Console.Clear();
                    Console.WriteLine("Byl zadán neplatný řádek. Zkuste to znovu:\n\n");
                    if (y >= 0) for (int r = 0; r < y; r++) Console.WriteLine(string.Join(" ", sudoku.Tabulka[r]));
                    y--; //načítání řídíme forem, musíme tedy řídicí proměnnou dekrementovat
                }
            }

            return sudoku;
        }

        /// <summary>
        /// Zobrazí daný text uživateli
        /// </summary>
        /// <param name="text">Text ke zobrazení</param>
        public void Display(string text)
        {
            Console.WriteLine(text);
        }
    }
}
