﻿namespace Sudoku
{
    /// <summary>
    /// Třída reprezentující řešič sudoku. Obsahuje metodu Solve, která vyřeší sudoku načtené komunikátorem.
    /// </summary>
    class Solver
    {
        /// <summary>
        /// Vyřeší sudoku načtené komunikátorem
        /// </summary>
        /// <param name="communicator">Komunikátor, který metoda použije k získání zadání a následnému výpisu získaných dat</param>
        public void Solve(Communicator communicator)
        {
            Sudoku zadani = communicator.Load(); //načítáme zadaní
            Sudoku reseni; //připravujeme si proměnnou pro řešení
            
            bool resitelne = BacktrackSudoku(zadani, out reseni); //rekurzivně řešíme sudoku, zjistíme, zdali je to vůbec možné

            if (resitelne) communicator.Display(reseni.ToString()); //při úspěšném vyřešení vypíšeme sudoku uživateli
            else communicator.Display("Algoritmus vyhodnotil, že pro toto zadání neexistuje řešení:\n\n" + zadani.ToString()); //při neúspěchu vrátíme sudoku uživateli spolu s hláškou, že jej není možné vyřešit
        }

        private bool BacktrackSudoku(Sudoku zadani, out Sudoku reseni) //metoda na vyřešení, předáme ji zadání a odkaz, pro zápis instance vyřešeného sudoku
        {
            reseni = (Sudoku)zadani.Clone(); //vytvoříme hlubokou kopii zadání

            int x = 0, y = 0;
            if (reseni.Tabulka[y][x] != 0) reseni.NextField(x, y, out x, out y, zadani); //založíme výchozí souřadnice, případně je posuneme na první nulovou pozici

            bool vyreseno = false, res = true; //je sudoku možné vyřešit? | Řídící proměnná hlavní smyčky while ("řeš!")

            while (res)
            {
                bool matches = false; //koresponduje doplňované číslo s ostatními?
                while (!matches)
                {
                    if (reseni.Tabulka[y][x] == 9) //pokud je číslo devět, vynulujeme jej a končíme
                    {
                        reseni.Tabulka[y][x] = 0;
                        break;
                    }

                    reseni.Tabulka[y][x]++; //pokud je číslo 0 - 8, inkrementujeme jej do té doby, než bude korespondovat či nepřesáhne hranici 9
                    matches = reseni.FieldMatches(x, y);
                    if (!matches && reseni.Tabulka[y][x] > 8)
                    {
                        reseni.Tabulka[y][x] = 0;
                        break;
                    }
                }

                if (reseni.Tabulka[y][x] != 0) //přejdeme na následující souřadnice
                {
                    res = reseni.NextField(x, y, out x, out y, zadani); //pokud jsme přesáhli hranici tabulky, přestáváme řešit a vyhodnocujeme výsledky
                    if (!res) vyreseno = true;
                }

                else //pokud ale číslo přesáhlo devítku, je nyní na nule. Nelze tedy pokračovat, ale musíme se vrátit zpět a zkusit vyšší číslo
                {
                    res = reseni.PreviousField(x, y, out x, out y, zadani);
                    if (!res) vyreseno = false; //pokud jsme přesáhli hranici tabulky, přestáváme řešit a vyhodnocujeme výsledky
                }
            }            

            return vyreseno;
        }
    }
}
