﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Sudoku
{
    class Sudoku
    {
        public Bunka[,] pole { get; set; }

        public void Vyres()
        {
            /*
                Algorytmus:
                    Před vkládáním čísel do buněk se nejdříve všem buňkám přiřadí indexy, čísla, která do nich jdou vložit, aniž by odporovala základním 
                pravidlům sudoku. 
                    Poté se hledá prázdná buňka z levého horního do pravého dolního rohu. Zkouší se do ní vložit její první index, pokud
                neodporuje pravidlům, hledá se další prázdná buňka, kam se znovu zkouší dosadit první index. Pokud však odporuje pravidlům,
                dosazuje se její další index a znovu se zkouší, jestli neodporuje pravidlům. Když nelze dosadit ani jeden z indexů, algorytmus
                se vrátí do předchozí buňky kam zkouší vložit další z její indexů. Toto se opakuje do té doby, dokud se sudoku nevyplní celé nebo
                do té doby, dokud algorytmus zjistí, že do první prázdné buňky nelze dosadit ani jeden z její indexů. V tomto případě nemá sudoku řešení.
                    Indexy by vlastně ani nemusely být uložené a místo zkoušení dosazování indexů by se zkoušely dosazovat čísla od 1 do 9. Ale vzhledem
                k tomu, že s indexy je to skoro o polovinu časově méně náročné, zvolil jsem tento způsob.
                    Doufám, že jsem to srozumitelně vysvětlil, pokud ne vyhledejte si 'sudoku backtracking'.
            */

            // Prirazovani indexu bunkam
            PriradIndexyBunkam();

            for (int i = 0; i < 9; i++)
            {
                for (int k = 0; k < 9; k++)
                {
                    if (pole[i, k].Cislo == -1) // Bunka je prazdna
                    {
                        foreach (sbyte index in pole[i, k].Indexy)
                        {
                            if (MuzePridatCislo(i, k, index)) // Zkouseni dosazeni indexu do bunky
                            {
                                pole[i, k].Cislo = index;
                                break;
                            }
                            else if (pole[i, k].Indexy.IndexOf(index) == pole[i, k].Indexy.Count - 1) // Index nelze dosadit do bunky
                            {
                                pole[i, k].Cislo = -1;

                                // Navigace do predchozi bunky
                                PosunUkazatelZpet(ref i, ref k);

                                break;
                            }
                        }
                    }
                    else if (!pole[i, k].Puvodni) // Bunka je sice plna, ale sudoku nema s temito cisly reseni
                    {
                        if (pole[i, k].Indexy.IndexOf(pole[i, k].Cislo) == pole[i, k].Indexy.Count - 1) // Cislo v bunce nelze zvetsit
                        {
                            pole[i, k].Cislo = -1;

                            // Navigace do predchozi bunky
                            PosunUkazatelZpet(ref i, ref k);
                        }
                        else
                        {
                            for (int index = pole[i, k].Indexy.IndexOf(pole[i, k].Cislo); index < pole[i, k].Indexy.Count; index++) // Dosazovani dalsich indexu
                            {
                                if (MuzePridatCislo(i, k, pole[i, k].Indexy[index])) // Zkouseni dosazeni indexu do bunky
                                {
                                    pole[i, k].Cislo = pole[i, k].Indexy[index];
                                    break;
                                }
                                else if (pole[i, k].Indexy.IndexOf(pole[i, k].Indexy[index]) == pole[i, k].Indexy.Count - 1) // Index nelze dosadit do bunky
                                {
                                    pole[i, k].Cislo = -1;

                                    // Navigace do predchozi bunky
                                    PosunUkazatelZpet(ref i, ref k);
                                    break;
                                }
                            }
                        }
                    }

                    // Algorytmum se nepodarilo najit reseni
                    if (i == -1)
                    {
                        Console.WriteLine("Algorytmům se nepodařilo najít řešení.");
                        return;
                    }
                }
            }
        }

        private void PosunUkazatelZpet(ref int i, ref int k)
        {
            do
            {
                k--;

                if (k == -1)
                {
                    k = 8;
                    i--;

                    if (i == -1) // Sudoku nema reseni
                        break;
                }
            }
            while (pole[i, k].Puvodni); // Pokud bylo 'Cislo' aktualni bunky zadane v zadani, hleda se predchozi bunka

            k--;
        }

        private bool MuzePridatCislo(int radek, int sloupec, int cislo)
        {
            // Kontrola pro radek a sloupec
            for (int l = 0; l < 9; l++)
            {
                // Kontrola pro radek
                for (int s = 0; s < 9; s++)
                {
                    if (pole[radek, s].Cislo == cislo)
                        return false;
                }

                // Kontrola pro sloupec
                for (int r = 0; r < 9; r++)
                {
                    if (pole[r, sloupec].Cislo == cislo)
                        return false;
                }
            }

            // Kontrola pole 3x3
            for (int bunka_radek = (radek / 3) * 3; bunka_radek < (radek / 3) * 3 + 3; bunka_radek++)
            {
                for (int bunka_sloupec = (sloupec / 3) * 3; bunka_sloupec < (sloupec / 3) * 3 + 3; bunka_sloupec++)
                {
                    if (bunka_radek == radek && bunka_sloupec == sloupec)
                        continue;

                    if (pole[bunka_radek, bunka_sloupec].Cislo == cislo)
                        return false;
                }
            }
            
            return true;
        }

        private void PriradIndexyBunkam()
        {
            for (int i = 0; i < 9; i++)
            {
                for (int k = 0; k < 9; k++)
                {
                    for (int index = 0; index < 9; index++)
                    {
                        if (MuzePridatCislo(i, k, index))
                            pole[i, k].Indexy.Add((sbyte)index);
                    }
                }
            }
        }

        public void Vypis()
        {
            for (int i = 0; i < 9; i++)
            {
                for (int k = 0; k < 9; k++)
                {
                    Console.Write((pole[i, k].Cislo + 1) + " ");
                }
                Console.WriteLine();
            }
        }
    }
}
