﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace sudoku
{
    public delegate void NumberChangedEventHandler(Number num);

    public class Sudoku
    {
        public event NumberChangedEventHandler NumberChanged;

        private SudokuData data = new SudokuData();     
        private bool loading = true, find_possibilities = false;

        public void ChangeNumber(Number num)
        {
            //tahle část je jen kvůli hezkosi vypsání ... :D
            if (NumberChanged != null && !find_possibilities)
                NumberChanged(num);

            //pokud se změní číslo ... musí se ve sloupi, řádku a čtverci(3x3) překontrolovat, kde jaká můžou být čísla
            if (!loading)
                number_changed(num.X, num.Y); 
        }

        public override string ToString()
        {
            string text = "";
            for (int y = 0; y < 9; y++)
            {
                for (int x = 0; x < 9; x++)
                {
                    text += data[x, y].ToString() + " ";
                }
                text += "\n";
            }
            return text;
        }

        public Sudoku(string data) //vytvoření 2D pole ze vstupu
        {
            string[] lines = data.Split('\n'); //rozdělení na řádky sudoky
            for (int y = 0; y < 9; y++)
            {
                string[] chars = lines[y].Split(' '); //rozdělení řádku na buňky
                for (int x = 0; x < 9; x++)
                {
                    string num = chars[x];
                    //pokud je buňka prázdná
                    if (num == "_")
                        this.data[x, y] = new Number(this, x, y);
                    else
                        this.data[x, y] = new Number(this, int.Parse(num), x, y);
                }
            }

            //nastavení buňek, co v nich může být za čísla
            for (int x = 0; x < 9; x++)
                for (int y = 0; y < 9; y++)
                    for (int i = 1; i < 10; i++)
                        this.data[x, y].SetNumber(i, check_position(x, y, i));
            loading = false;
        }

        private void number_changed(int start_x, int start_y)
        {
            for (int i = 1; i < 10; i++)
            {
                //procházení čtverce(3x3) a doplňuje, jáká čísla můžou být v dané buňce
                Number[,] block = data.GetBlock(start_x, start_y);
                for (int pos_x = 0; pos_x < 3; pos_x++)
                {
                    for (int pos_y = 0; pos_y < 3; pos_y++)
                    {
                        int x = block[pos_x, pos_y].X, y = block[pos_x, pos_y].Y;
                        data[x, y].SetNumber(i, check_position(x, y, i));
                    }
                }

                //procházení řádku a doplňuje, jáká čísla můžou být v dané buňce
                for (int y = 0; y < 9; y++)
                    data[start_x, y].SetNumber(i, check_position(start_x, y, i));

                //procházení sloupce a doplňuje, jáká čísla můžou být v dané buňce
                for (int x = 0; x < 9; x++)
                    data[x, start_y].SetNumber(i, check_position(x, start_y, i));
            }
        }

        //kontroluje zda na pozici může být konkrétní číslo
        public bool check_position(int x, int y, int num)
        {
            //pokud je buňka v sudoce prázdná a řádek, sloupec, ani čtverec(3x3) neobsahuje dané číslo ...
            return data[x, y].IsEmpty && !data.BlockContainsNumber(x, y, num) && !data.RowContainsNumber(y, num)
                 && !data.ColumnContainsNumber(x, num);
        }

        public bool Solve()
        {
            //řešení sudoku

            //klasickým způsobem
            bool change = false;
            do
            {
                //pokud nějaká metoda přidala číslo, bude se cyklus opakovat
                change = exclusion() || rows() || columns() || blocks();
            } while (change);

            //všechny možnosti
            all_possibilities();
            return (check_sudoku());
        }

        //hledání všech zbývajících možností
        private void all_possibilities()
        {
            SudokuData backup_data = data; //záloha aktuální sudoky
            find_possibilities = true;

            List<Number> nums = new List<Number>(); //seznam nedoplněných pozic
            Dictionary<string, int> indexes = new Dictionary<string, int>();//index právě zkoušené možnosti
            Dictionary<string, List<int>> nums_at_index = new Dictionary<string, List<int>>(); //seznam možnosí 
            int count = 0; //součet možností z každého pole

            //projití všech prázdných pozic a nastavení proměnných
            foreach (Number n in data.Data)
            {
                if (n.IsEmpty)
                {
                    nums.Add(n);
                    string key = n.X.ToString() + "," + n.Y.ToString();
                    indexes.Add(key, 0);
                    nums_at_index.Add(key, new List<int>());
                    for (int i = 1; i < 10; i++)
                        if (n.Numbers(i))
                            nums_at_index[key].Add(i);
                    count += nums_at_index[key].Count;
                }
            }

            //zkoušení všech možností
            int possibility = 0;
            for (int i = 0; i < nums.Count; i++)
            {
                possibility++;
                string key = nums[i].X.ToString() + "," + nums[i].Y.ToString(); //pozice x,y
                if (indexes[key] == nums_at_index[key].Count) //pokud se na téhle pozic vyzkoušeli všechny možnosti ...
                {
                    //této pozici se nastaví 1. možnost
                    indexes[key] = 0;
                    i--;

                    //a na předchozí pozici se posunume k další možnosti
                    key = nums[i].X.ToString() + "," + nums[i].Y.ToString();
                    indexes[key]++;

                    //smazaní možnosti
                    data[nums[i].X, nums[i].Y].Value = 0;
                    i--;
                }
                else
                {
                    //vyzkoušení všech ještě nezkoušených možnostína této pozici
                    for (int j = indexes[key]; j < nums_at_index[key].Count; j++)
                    {
                        if (nums[i].Numbers(nums_at_index[key][j])) //pokud tato pozice může obsahovat tuto možnost
                        {
                            //možnost se uloží a ukončí se hledání na této pozici
                            indexes[key] = j;
                            data[nums[i].X, nums[i].Y].Value = nums_at_index[key][j];
                            break;
                        }
                        else
                        {
                            indexes[key]++;
                            if (indexes[key] == nums_at_index[key].Count) //pokud se na téhle pozic vyzkoušeli všechny možnosti ...
                            {
                                //této pozici se nastaví 1. možnost
                                indexes[key] = 0;
                                i--;

                                //a na předchozí pozici se posunume k další možnosti
                                key = nums[i].X.ToString() + "," + nums[i].Y.ToString();
                                indexes[key]++;

                                //smazaní možnosti
                                data[nums[i].X, nums[i].Y].Value = 0;
                                i--;

                                break;
                            }
                        }
                    }
                }

                //pokud se prošlo tolik možností (pošet volných polí * součet možností z každého pole) a
                //řešení se pořád nenašlo ... je něco špatně a nekonečný cyklus je třeba ukončit
                if (possibility == nums.Count * count)
                    break;
            }

            find_possibilities = false;
            if (!check_sudoku())
            {
                data = backup_data;
            }
            else
            {
                //doplnění správných možností
                foreach (Number n in nums)
                    data[n.X, n.Y].Value = data[n.X, n.Y].Value;
            }
        }

        //kontrolování správnosti sudoky
        private bool check_sudoku()
        {

            //procházení všech pozic
            for (int x = 0; x < 9; x++)
            {
                for (int y = 0; y < 9; y++)
                {
                    //pokud je pozice prázdná ... sudoka není správně vyplněná
                    if (data[x, y].IsEmpty)
                        return false;

                    //procházení čtverce(3x3) 
                    Number[,] block = data.GetBlock(x, y);
                    for (int block_x = 0; block_x < 3; block_x++)
                    {
                        for (int block_y = 0; block_y < 3; block_y++)
                        {
                            //pokud je pozice prázdná ... sudoka není správně vyplněná
                            if (block[block_x, block_y].IsEmpty)
                                return false;

                            //pokud je číslo ve čtverci(3x3) ještě jednou .... sudoka není správně vyplněná
                            if (block[block_x, block_y].X != x && block[block_x, block_y].Y != y
                                && block[block_x, block_y].Value == data[x, y].Value)
                                return false;
                        }
                    }

                    //procházení sloupce
                    for (int column = 0; column < 9; column++)
                    {
                        //pokud je pozice prázdná ... sudoka není správně vyplněná
                        if (data[column, y].IsEmpty)
                            return false;

                        //pokud sloupec obsahuje číslo ještě jednou ... sudoka není správně vyplněná
                        if (column != x && data[column, y].Value == data[x, y].Value)
                            return false;
                    }

                    //procházení řádku
                    for (int row = 0; row < 9; row++)
                    {
                        //pokud je pozice prázdná ... sudoka není správně vyplněná
                        if (data[x, row].IsEmpty)
                            return false;

                        //pokud řádek obsahuje číslo ještě jednou ... sudoka není správně vyplněná
                        if (row != y && data[x, row].Value == data[x, y].Value)
                            return false;
                    }
                }
            }

            return true; //sudoka je správně vyplněná
        }

        //použije se vylučovací metoda na každé volné pole v sudoce
        private bool exclusion()
        {
            bool change = false, ex_change = false;
            do
            {
                ex_change = false;
                for (int x = 0; x < 9; x++)
                {
                    for (int y = 0; y < 9; y++)
                    {
                        if (pos_exclusion(x, y))
                        {
                            change = true;
                            ex_change = true;
                        }
                    }
                }
            } while (ex_change);
            return change;
        }

        //doplňuje čísla do čtverců(3x3)
        private bool blocks()
        {
            bool change = false;
            for (int block_x = 0; block_x < 3; block_x++)
            {
                bool block_change = false;
                
                for (int block_y = 0; block_y < 3; block_y++)
                {
                    //pole kam se ukládá kde to číslo může být
                    Dictionary<int, string> number_in_block = new Dictionary<int, string>(); //<cislo | "pozice x1 ve čtverci, pozice y1 ... x2 ..."> 

                    Number[,] block = data.GetBlock(block_x * 3, block_y * 3); //vrací čtverec z pozice v sudoce
                    for (int i = 1; i < 10; i++)
                    {
                        for (int x = 0; x < 3; x++)
                        {
                            for (int y = 0; y < 3; y++)
                            {
                                int pos_x = x + block[0, 0].X, pos_y = y + block[0, 0].Y;
                                if (data[pos_x, pos_y].Numbers(i))
                                {
                                    if (number_in_block.ContainsKey(i))
                                        number_in_block[i] += ";" + x + "," + y;
                                    else
                                        number_in_block.Add(i, x + "," + y);
                                }
                            }
                        }

                        if (number_in_block.ContainsKey(i))
                        {
                            string[] split = number_in_block[i].Split(';');
                            if (split.Length == 1)
                            {
                                int x = int.Parse(split[0].Split(',')[0]), y = int.Parse(split[0].Split(',')[1]);
                                data[x + block[0, 0].X, y + block[0, 0].Y].Value = i;
                                change = true;
                                block_change = true;
                            }
                        }
                    }
                }

                if (block_change) block_x = -1; //pokud se přidalo nějaké číslo, všechny čtverce(3x3) se projedou znovu
            }
            return change;
        }

        //doplňuje čísla do sloupce
        private bool columns()
        {
            bool change = false;
            for (int column = 0; column < 9; column++) //procházení všech řádků sudoky
            {
                bool column_change = false;

                for (int i = 1; i < 10; i++) //hledání čísel<1; 9> na každý sloupec
                {
                    if (!data.ColumnContainsNumber(column, i)) //pokud sloupec číslo ještě neobsahuje, bude se číslo hledat
                    {
                        //hledání čísla v sloupci

                        bool[] pos = new bool[9]; //pole kde všude číso může být
                        for (int row = 0; row < 9; row++) //procházení všech buněk sloupce
                            pos[row] = data[column, row].Numbers(i);

                        if (count(pos, true) == 1) //pokud číslo v sloupci může být jen na jednom místě, zapíše se 
                        {
                            int y = Array.IndexOf(pos, true); //vrací pozici ve sloupci
                            data[column, y].Value = i;
                            change = true;
                            column_change = true;
                        }
                    }
                }

                if (column_change) column = -1; //pokud se přidalo nějaké číslo, všechny sloupce se projedou znovu
            }
            return change;
        }

        //doplňuje číslo do řádku
        private bool rows()
        {
            bool change = false;
            for (int row = 0; row < 9; row++) //procházení všech řádků sudoky
            {
                bool row_change = false;

                for (int i = 1; i < 10; i++) //hledání čísel<1; 9> na každý řádek
                {
                    if (!data.RowContainsNumber(row, i)) //pokud řádek číslo ještě neobsahuje, bude se číslo hledat
                    {
                        //hledání čísla v řádku

                        bool[] pos = new bool[9]; //pole kde všude číso může být
                        for (int column = 0; column < 9; column++) //procházení všech buněk řádku
                            pos[column] = data[column, row].Numbers(i); 

                        if (count(pos, true) == 1) //pokud číslo v řádku může být jen na jednom místě, zapíše se 
                        {
                            int x = Array.IndexOf(pos, true); //vrací pozici na řádku
                            data[x, row].Value = i;
                            change = true;
                            row_change = true;
                        }
                    }
                }

                if (row_change) row = -1; //pokud se přidalo nějaké číslo, všechny řádky se projedou znovu
            }
            return change;
        }

        //vrací kolikrát se prvek vyskytuje v poli
        private int count(bool[] array, bool find)
        {
            int count = 0;
            for (int i = 0; i < array.Length; i++)
                if (array[i] == find)
                    count++;
            return count;
        }

        //vylučovací metoda
        //zjistí zda pozice může obsahovat více jak jedno číslo, pokud ne ... doplní ho
        private bool pos_exclusion(int pos_x, int pos_y)
        {
            bool change = false;
            bool[] pos = new bool[9];
            for (int i = 1; i < 10; i++)
                pos[i - 1] = data[pos_x, pos_y].Numbers(i);

            if (count(pos, true) == 1) //pokud na pozici může být jen jedno číslo
            {
                int num = Array.IndexOf(pos, true); //vrací číslo
                data[pos_x, pos_y].Value = num + 1;
                change = true;
            }

            return change;
        }
    }
}
